/** 
 *   Encoders: Control over iar bus (server).
 *
 *   (c) Copyright 205 Federico Bareilles <fede@iar.unlp.edu.ar>,
 *   Instituto Argentino de Radio Astronomia (IAR).
 *
 *   This program is free software; you can redistribute it and/or
 *   modify it under the terms of the GNU General Public License as
 *   published by the Free Software Foundation; either version 2 of
 *   the License, or (at your option) any later version.
 *     
 *   The author does NOT admit liability nor provide warranty for any
 *   of this software. This material is provided "AS-IS" in the hope
 *   that it may be useful for others.
 *
 *   TODO: 
 *        exit on encoders_core();
 **/
#define _GNU_SOURCE

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include <fcntl.h>
#include <sys/ioctl.h>
#include <syslog.h>
#include <signal.h>

#include "my_stdio.h"
#include "iar_bus.h"
#include "antenna_ib_pk.h"

#include "dwt.h"
#include "dwt_ib_server.h"


int dwt_done = 0;

/*****************************************************************************
 * local functions:
 **/

static ssize_t soft_read( int fd, void *buf, size_t count)
{
        fd_set rfds;
        int fd_max = fd + 1;
        int retval;
	struct timeval tv;

	while( 1 ) {
		tv.tv_sec = 10;
		tv.tv_usec = 0;
		FD_ZERO(&rfds);
		FD_SET(fd, &rfds);
		retval = select(fd_max, &rfds, NULL, NULL, &tv);
		if (retval) {
			if( FD_ISSET(fd, &rfds) ) {
				retval = read(fd, buf, count);
#ifdef DEBUG_MODE
				if (verbose_flag & PINFO) {
					int __i;
					PVERB(PINFO, 
					      "# dwt_ib_server.c read <- ");
					for(__i=0;__i<retval;__i++)
						PVERB(PINFO, "%02x ",
						      *(((u_char *)buf)+__i) );
					PVERB(PINFO, "\n");
				}
#endif
				return retval;
			}
		}
	}		
	
	return -1; /* never */
}
	


static int dwt_get_loop(int fd_dwt, char *data)
{
	char *buff = NULL;

#if 0
        if ( wake_recv(fd_dwt) == 0) {
                SYSLOG(LOG_ERR,"wake_recv dwt\n");
                return -1;
        }
#endif
	buff = GET_loop(fd_dwt);
        if(buff){
		memcpy(data, buff, LOOP_LEN);
		free(buff);
        }

        return 0;
}


static int dwt_cmd(int fd, int fd_bus, struct antenna_packet *last_loop)
{
	struct ib_packet pk;
	struct antenna_packet *anp;
	int ret;
	int val;
	int len;
	int send_ret;
	int send_data;
	char *buff = NULL;

	anp = (struct antenna_packet *)&pk.data;
	send_ret = 1;
	send_data = 0;
	memset(&pk, 0, sizeof(struct ib_packet));
	ret = soft_read(fd_bus, &pk, sizeof(struct ib_packet_head) );
	if( pk.h.size > 0) {
		ret += soft_read(fd_bus, &pk.data, pk.h.size );
		val = atoi(pk.data);
	}
	switch (anp->h.cmd) {
		/* FIXME: estan mal implementados en dwt_hw.c */

	case WCM_LOOP:
		ret = ibus_send_data_to(fd_bus, pk.h.addr, last_loop,
				  sizeof(struct antenna_packet_head) + 
				  last_loop->h.size);
		break;
	case WCM_GETTIME:
		send_data = 1;
		buff = (char *) CMD_gettime(fd);
		anp->h.size = sizeof(struct tm);
		ret = -1;
		break;
	case WCM_TEST:
		send_data = 1;
		buff = (char *) CMD_test(fd);
		anp->h.size = 8;
		ret = -1;
		break;
	case WCM_RXCHECK:
		send_data = 1;
		buff = (char *) CMD_rxcheck(fd);
		anp->h.size = 35;
		ret = -1;
		break;
	case WCM_VER:
		send_data = 1;
		buff = (char *) CMD_ver(fd);
		anp->h.size = 19;
		ret = -1;
		break;
	case WCM_WRD: 
		ret = CMD_wrd(fd);
		break;
	case WCM_SETTIME:
		wake_recv(fd);
		ret = CMD_settime(fd);
		break;
	default:
		SYSLOG(LOG_ERR,
		       "dwt core: bad command (0x%x) from 0x%x",
		       anp->h.cmd, pk.h.addr);
	case SCM_INFO: /* ignored */
		send_ret = 0;
		break;
	}
	if (send_data) {
		if (buff != NULL) {
			PVERB(PINFO,"# sendind data: size = %d\n",
			      anp->h.size);
			memcpy(anp->data, buff, anp->h.size);
			ret = ibus_send_data_to(fd_bus, pk.h.addr, anp,
					sizeof(struct antenna_packet_head) + 
						anp->h.size);
			free(buff);
		}
	}

	if ( send_ret) {
		/* Send ret value: */
		PVERB(PINFO,"# sendind ret: %d\n", ret);
		len = anp->h.size = sprintf(anp->data, "%d",  ret);
		len += sizeof(struct antenna_packet_head);
		anp->h.cmd = CM_RETURN;
		ibus_send_data_to(fd_bus, pk.h.addr, anp, len);
	}

	return 0;
}

/*
 * local functions END.
 *****************************************************************************/


int dwt_core(int fd, int fd_bus)
{
	struct antenna_packet ap;
	int data_len = sizeof(struct antenna_packet_head) + LOOP_LEN;
	fd_set rfds;
        int fd_max = fd_bus + 1;
        int retval;
	struct timeval tv;

	ap.h.cmd = WCM_LOOP;
	ap.h.size = LOOP_LEN;
	
	if ( wake_recv(fd) == 0) {
                SYSLOG(LOG_ERR,"wake_recv dwt\n");
                return -1;
	}

	while(!dwt_done){
		if (dwt_get_loop(fd, (char *) &ap.data ) == 0) {
			ibus_send_data( fd_bus, &ap, data_len );
		}
		tv.tv_sec = 0;
		tv.tv_usec = 100000;
		do {
			FD_ZERO(&rfds);
			FD_SET(fd_bus, &rfds);
			retval = select(fd_max, &rfds, NULL, NULL, &tv);
			if (retval) {
				printf("Process command.\n");
				dwt_cmd(fd, fd_bus, &ap);
			}
		}while(retval);
	}

	return 0;
}




