/** 
 *   
 *
 *   (c) Copyright 2005 Federico Bareilles <fede@iar.unlp.edu.ar>,
 *   Instituto Argentino de Radio Astronomia (IAR).
 *
 *   This program is free software; you can redistribute it and/or
 *   modify it under the terms of the GNU General Public License as
 *   published by the Free Software Foundation; either version 2 of
 *   the License, or (at your option) any later version.
 *     
 *   The author does NOT admit liability nor provide warranty for any
 *   of this software. This material is provided "AS-IS" in the hope
 *   that it may be useful for others.
 *
 **/

#ifndef _DWT_H
#define _DWT_H

#include <unistd.h>
#include <time.h>
#include <sys/types.h>


#define LOOP_LEN 99
#define DWT_PORT "/dev/vantagepro"


/* Wind speed  */
#define U_KPH 1		/* Kilometers per hour */
#define U_MPH 2		/* Miles per hour */
#define U_MPS 3		/* Meters per second */

/* Temperature */
#define U_FAH 4		/* Fahrenheit */
#define U_CEL 5		/* Celcius */

/* Atmospheric pressure */
#define U_HPA 6		/* hPa */
#define U_IHG 7		/* Inches of HG */

/* Rainfall */
#define U_MIL 8		/* Millimeters */
#define U_INC 9		/* Inches */
#define U_CLK 10	/* Clicks */

/* Default output units to metric */
#define DEF_WS U_KPH
#define DEF_TM U_CEL
#define DEF_AP U_HPA
#define DEF_RF U_MIL

typedef struct dmap {
        char	*tag;
	ushort	ofs, len;	
        void	*(*fn)(char *, ushort, ushort, ushort);
	char	type;
	char	*desc;
	ushort	conv;
} map; 

extern int map_items;

int open_dwt(char *tty);
int close_dwt(int fd);
ushort wake_recv(int fd);
char *GET_loop(int fd);

/* defined in dwt_utils.c: */
void *GET_temp(char *buf, ushort ofs, ushort len, ushort conv);
void *GET_barometer(char *buf, ushort ofs, ushort len, ushort conv);
void *GET_humidity(char *buf, ushort ofs, ushort len, ushort conv);
void *GET_rain(char *buf, ushort ofs, ushort len, ushort conv);
void *GET_windspeed(char *buf, ushort ofs, ushort len, ushort conv);
void *GET_winddirection(char *buf, ushort ofs, ushort len, ushort conv);
void *GET_winddegree(char *buf, ushort ofs, ushort len, ushort conv);
void *GET_FIELD(char *fld, char *bp, ushort conv);
void *GET_dewpoint(char *buf, ushort ofs, ushort len, ushort conv);
void *GET_windchill(char *buf, ushort ofs, ushort len, ushort conv);
void *GET_heatindex(char *buf, ushort ofs, ushort len, ushort conv);
void *GET_time(char *buf, ushort ofs, ushort len, ushort conv);
void CMD_list(int fd);
void CMD_frmt(int fd, char *frmt, ushort n, ushort d);
void CMD_tags();
ushort get_cmd(char *u);
ushort valid_cmd(char *cmd);
char *dwt_type(int type);
int dwt_make_hash_table(void);
int dwt_destroy_hash_table(void);

/* defined in dwt_hw.c and dwt_ib_client.c: */
int CMD_settime(int fd);
struct tm *CMD_gettime(int fd);
char *CMD_test(int fd);
char *CMD_rxcheck(int fd);
char *CMD_ver(int fd);
int CMD_wrd(int fd);


#endif /* _DWT_H */
