/** 
 *   FIXME ....
 *
 *   (c) Copyright 2004 Federico Bareilles <fede@iar.unlp.edu.ar>,
 *   Instituto Argentino de Radio Astronomia (IAR).
 *
 *   This program is free software; you can redistribute it and/or
 *   modify it under the terms of the GNU General Public License as
 *   published by the Free Software Foundation; either version 2 of
 *   the License, or (at your option) any later version.
 *     
 *   The author does NOT admit liability nor provide warranty for any
 *   of this software. This material is provided "AS-IS" in the hope
 *   that it may be useful for others.
 *
 *   TODO: ???
 *      *) validation (getpwent? lh_stats ?).
 *      *) Options at startup.
 **/

#define NOFORK
#define NONETWORK

#ifndef NONETWORK
#  define NETWORK
#endif

#ifndef NODAEMON
#  define DAEMON
#endif

#include <errno.h> 
#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <getopt.h>
#include <sys/types.h>
#include <sys/resource.h>
#include <sys/wait.h>
#include <sys/socket.h>

#include "encoders.h"
#include "command.h"
#include "socket_srv.h"
#include "socket_client.h"
#include "iar.h"
#include "my_stdio.h"
#include "anemometer.h"
#include "coordinate.h"
#include "iar_bus.h"
#include "survey.h"


int start_survey = 0;

#undef SYSLOG
#define SYSLOG(type, fmt, args...)

#undef PERROR
#define PERROR(s) { PVERB("[ERROR: %s: %s]\n", (s),strerror(errno));\
                    SYSLOG(LOG_ERR,"%s: %s",(s), strerror(errno));\
                  }


void fireman(int s)
{ 
	int wstatus;

	while(wait3(&wstatus, WNOHANG, NULL) > 0);
}


int cartel(int fd, char *prefix)
{
	fd_printf(fd, "%sIAR Antenna position system.\r\n", prefix);
	fd_printf(fd, "%s\r\n", prefix);

	return 0;
}


int main(int argc, char *argv[]) 
{
	int fd_socket, fd_con;
	int fd_track, fd;
	int pid, pid_bus;
	int ra = RA_CASE;
	int dec = DEC_CASE;
	//char *cp;
	int op;
	static struct option  const  longopts[] =
	{
		{"help", 0, 0, 'h'},
		{"verbose", 0, 0, 'v'},
		{0,  0,  0,  0}
	};
	
	opterr =  0;
	while  ((op = getopt_long(argc, argv, "vh",longopts, 0)) != EOF)
		switch( op ){
		case 'v':
			verbose_flag = (verbose_flag << 1) | 1;
			break;
		case 'h':
			//usage();
			//help();
			exit(-1); 
		default:
			//usage();
			exit(-1);
		}
	
	if ( (pid=fork()) == 0 ) {
		iar_bus_start(TRACK_POS_SOCK_NAME);
		exit(0);
	}

	usleep(1000); /* wait for bus startup */

	/* encoders: registry on bus */
	fd = call_socket_local(TRACK_POS_SOCK_NAME);
	if (fd < 0 ) exit(1);
	if ( ibus_set_addr(fd, ANTENNA_BUS_ADDR) != ANTENNA_BUS_ADDR) {
		PVERB(PERR,"Antenna bus addr falled.");
		exit(1);
	}
	if ( (pid_bus = fork()) == 0 ) {
		ibus_rw( fd, IB_DEV_WRITE); /* set write only */
		ibus_on( fd );
		if ( encoders_open(ra, dec, fd) == -1){
			PERROR("main->encoders");
			exit(1);
		}
		ibus_remove( fd );
		close( fd );
		exit(0);
	}
	close( fd );

	/* survey: registry on bus */
	fd = call_socket_local(TRACK_POS_SOCK_NAME);
	if (fd < 0 ) exit(1);
	if ( ibus_set_addr(fd, SURVEY_BUS_ADDR) != SURVEY_BUS_ADDR) 
		exit(1);
	if ( (pid=fork()) == 0 ) {
		ibus_on( fd );
		if ( survey_main(fd) < 0 ){
			perror("main->survey");
			exit(1);
		}
		ibus_remove( fd );
		close( fd );
		exit(0);
	}
	close( fd );

#ifdef DAEMON
	if ( (pid=fork()) != 0 ) {
		exit(0);
	}
#endif


#ifdef NETWORK
        if ( (fd_socket = socket_init( ANTENNA_TCP_PORT )) < 0) {
		PERROR("main->socket_init");
                exit(1);
        }
#endif
	if (signal(SIGCHLD, fireman) == SIG_ERR) {/* this eliminates zombies */
		PERROR("main->Signal Failed (SIGCHLD).");
		exit(1);
	}
	/*
	if (signal(SIGINT, cbrk_handler) == SIG_ERR) {
		PERROR("Signal Failed (SIGINT).");
		EXIT(0);
	}
	if (signal(SIGTERM, cbrk_handler) == SIG_ERR) {
		PERROR("Signal Failed (SIGTERM).");
		EXIT(0);
	}
	*/

	for(;1;){
#ifdef NETWORK
		/* get a connection */
		if ((fd_con = socket_wait(fd_socket)) < 0) { 
			if (errno == EINTR) 
				/* EINTR might happen on accept(), */
				continue; /* try again */
			PERROR("main->accept"); /* bad */
			exit(1);
		}
#else
		fd_con = 0;
#endif

#ifdef NOFORK
		pid = 0;
#else
		pid = fork();
#endif
		switch(pid) {            /* try to handle connection */
		case -1 :                /* bad news.  scream and die */
			PERROR("main->fork");
			close(fd_socket);
			close(fd_con);
			exit(1);
		case 0 : /* we're the child, do command proses */
			cartel(fd_con,"# ");
			fd_track = call_socket_local(TRACK_POS_SOCK_NAME);
			command_main(fd_con, fd_con, fd_track);
			close(fd_con);
			close(fd_track);
			exit(0);
		default :                 /* we're the parent so look for */
			close(fd_con);    /* another connection */
			continue;
		}
	}

	return 0;
}


